import argparse
import pathlib


def parse_args() -> argparse.Namespace:
    # Get command line args
    parser = argparse.ArgumentParser(
        prog="snvis",
        description="Create and view a graph representation of a social network.")
    parser.add_argument(
        "-n",
        type=int,
        help="number of the column containing the name (default: 0)",
        metavar="COLUMN",
        default=0
    )
    parser.add_argument(
        "-c",
        type=int,
        help="number of the column containing the connections (default: 1)",
        metavar="COLUMN",
        default=1
    )
    parser.add_argument(
        "-o",
        type=pathlib.Path,
        help="name of the output file (default: 'graph.svg')",
        metavar="FILE",
        default="graph.svg"
    )
    parser.add_argument(
        "-q",
        help="stop logging to stdout",
        action="store_true"
    )
    parser.add_argument(
        "input",
        type=pathlib.Path,
        help="tab separated values file",
        metavar="SPREADSHEET"
    )
    parser.add_argument(
        "-v",
        "--view",
        help="view the output using default system tool",
        action="store_true"
	)
    args = parser.parse_args()
    return args



import sys


class color:
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'


class logger:
    def __init__(self, quiet: bool):
        self.quiet = quiet

    def status_msg(self, message: str):
        if not self.quiet:
            print(color.BOLD + color.BLUE + "[STATUS]" + color.END, message)

    def error_msg(self, message: str):
        print(color.BOLD + color.RED + "[ERROR]" +
              color.END, message, file=sys.stderr)

    def custom_msg(self, label: str, message: str):
        if not self.quiet:
            print(color.BOLD + color.GREEN +
                  "[" + label + "]" + color.END, message)



#!/usr/bin/env python

from setuptools import setup, find_packages

with open("README.md") as readme:
    setup(
        name="snvis",
        version="0.1.7",
        description="Social network visualising tool",
        long_description=readme.read(),
        long_description_content_type="text/markdown",
        url="https://github.com/Callum-Irving/snvis",
        author="Callum Irving",
        author_email="callum.irving04@gmail.com",
        license="MIT",
        classifiers=[
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 3.9",
        ],
        package_dir={"": "."},
        packages=find_packages(),
        include_package_data=True,
        install_requires=["thefuzz >= 0.19.0", "python-igraph >= 0.9.8",
                          "python-Levenshtein>=0.12.2", "cairocffi>=1.3.0"],
        entry_points={"console_scripts": ["snvis = snvis.__main__:main"]}
    )



import argparse
from operator import itemgetter
import os
import subprocess
from shutil import which

import igraph
from thefuzz import fuzz

from snvis.log import logger as logger_generator


def main(args: argparse.Namespace) -> int:
    logger = logger_generator(args.q)

    # Read spreadsheet
    logger.status_msg("Parsing file")

    rows: dict[str, list[str]] = {}

    with open(args.input) as f:
        labels = f.readline().strip().split("\t")
        if not args.n < len(labels):
            logger.error_msg(f"Column number '{args.n}' is too large")
            return 1
        elif not args.c < len(labels):
            logger.error_msg(f"Column number '{args.c}' is too large")
            return 1
        elif args.n == args.c:
            logger.error_msg(
                f"Name and connections columns cannot be the same")
            return 1

        for row, line in enumerate(f.readlines()):
            cols = line.strip().split("\t")

            try:
                name = cols[args.n].strip()
            except IndexError:
                logger.error_msg(f"Name column at row {row+2} is empty")
                return 1

            try:
                cons = cols[args.c].split(",")
                cons = [x.strip() for x in cons]
            except IndexError:
                cons = []

            rows[name] = cons

    # Name matching
    def check_name_match(name):
        others = list(rows.keys())
        if name in others:
            return name

        ratios = [(fuzz.ratio(name, to_check), to_check)
                  for to_check in others]

        closest_match = max(ratios, key=itemgetter(0))
        if 100 > closest_match[0] > 80:
            logger.custom_msg(
                "NAME MATCHER", f"{name} is similar to {closest_match[1]}")
            return closest_match[1]

        return None

    for name, cons in rows.items():
        for i, con in enumerate(cons):
            closest_match = check_name_match(con)
            if closest_match == None:
                logger.custom_msg(
                    "NAME MATCHER",
                    f"Unknown name in connections to {name}: {cons[i]}")
                cons.pop(i)
            else:
                cons[i] = closest_match
        rows[name] = cons

    # Generate graph
    logger.status_msg("Generating graph")
    people = igraph.Graph()
    people.add_vertices(len(rows))
    people.vs["label"] = list(rows.keys())
    for name, cons in rows.items():
        edge_from = people.vs.find(label=name).index
        for con in cons:
            edge_to = people.vs.find(label=con).index
            people.add_edge(edge_from, edge_to)

            # Stop duplicates
            if name in rows[con]:
                rows[con].remove(name)

    # Plot graph
    logger.status_msg("Writing graph to file")
    layout = people.layout("kk")
    igraph.plot(people, str(args.o), margin=30, layout=layout)

    # Show graph using default program from system
    if not args.view:
        return

    if os.sys.platform.startswith("linux"):
        if which("xdg-open"):
            logger.status_msg("Opening file with xdg-open")
            subprocess.run(["xdg-open", str(args.o)],
                           stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        else:
            logger.error_msg("xdg-open is not installed")
            return 1
    # TODO: Following are UNTESTED
    elif os.sys.platform == "win32":
        logger.status_msg("Opening SVG with default program")
        subprocess.run(["start", str(args.o)])
    elif os.sys.platform == "darwin":
        logger.status_msg("Opening SVG with default program")
        subprocess.run(["open", str(args.o)])
    else:
        logger.error_msg(f"Unsupported platform: {os.sys.platform}")
        return 1

    logger.status_msg("Exited succesfully")
    return 0
